#define  WPCAP
#define  HAVE_REMOTE
#include "pcap.h"
#include "Iphlpapi.h"

#pragma comment(lib, "wpcap.lib")
#pragma comment(lib, "Iphlpapi.lib")
#pragma comment(lib, "ws2_32.lib")

PIP_ADAPTER_INFO getSystemAdaptersInfo()
{
  PIP_ADAPTER_INFO allIPAdapters = NULL;
  ULONG outBufLen = 0;
  DWORD res = GetAdaptersInfo(allIPAdapters, &outBufLen);
  if (res == ERROR_BUFFER_OVERFLOW)
  {
    allIPAdapters = (PIP_ADAPTER_INFO)malloc(outBufLen);
    res = GetAdaptersInfo(allIPAdapters, &outBufLen);
  }
  if (res != ERROR_SUCCESS)
  {
    printf("GetAdaptersInfo return %d(0x%x)", res, res);
    return NULL;
  }
  return allIPAdapters;
}

pcap_if_t* getWinPcapAdaptersInfo()
{
  pcap_if_t* alldevs;
  char errbuf[PCAP_ERRBUF_SIZE];

  /* Retrieve the device list from the local machine */
  if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL /* auth is not needed */, &alldevs, errbuf) == -1)
  {
    printf("Error in pcap_findalldevs_ex: %s\n", errbuf);
    return NULL;
  }

  return alldevs;
}

pcap_if_t* findBestAdapter(PIP_ADAPTER_INFO allIPAdaptersInfo, pcap_if_t* alldevs)
{
  DWORD  bestIfIndex;
  IPAddr ipAddr = htonl(0x08080808); // "8.8.8.8" is well-knowon Google DNS Server IP.
  DWORD res = GetBestInterface(ipAddr, &bestIfIndex);
  if (res != NO_ERROR)
  {
    printf("GetBestInterface return %d", res);
    return NULL;
  }

  PIP_ADAPTER_INFO p = allIPAdaptersInfo;
  while (p != NULL)
  {
    if (bestIfIndex == p->Index) break;
    p = p->Next;
  }
  if (p == NULL)
  {
    printf("Can not find appropriate system adapter\n");
    return NULL;
  }

  pcap_if_t *d = alldevs;
  while (d != NULL)
  {
    if (strstr(d->name, p->AdapterName) != NULL)
      break;
  }
  if (d == NULL)
  {
    printf("Can not find appropriate winpcap adapter\n");
    return NULL;
  }
  return d;
}

int main()
{
  //
  // Get system adapters information
  //
  PIP_ADAPTER_INFO allIPAdapters = getSystemAdaptersInfo();
  if (allIPAdapters == NULL) return 0;

  //
  // Get wincap adapters information
  //
  pcap_if_t* alldevs = getWinPcapAdaptersInfo();
  if (alldevs == NULL)
  {
    free(allIPAdapters);
    return 0;
  }

  //
  // Find best winpcap adapter information
  //
  pcap_if_t* d = findBestAdapter(allIPAdapters, alldevs);
  if (d != NULL)
  {
    printf("best winpcap adapter is %s(%s)\n", d->description, d->name);
  }

  free(allIPAdapters);
  pcap_freealldevs(alldevs);
}